#
#  Copyright 1996-2014 Jim Pirzyk / pirzyk.org
#  All rights reserved.
#
#  Security::util::uniq - Return only the unique elements in an array
#
package Security::util::uniq;

use Exporter;
use strict;
use warnings;

our @ISA = qw(Exporter);
our @EXPORT_OK = qw( uniq uniq_obj );

sub uniq {
	my (@a) = @_;
	my (%b);

	map { $b{$_} = 1 } @a;

	return keys %b;
}

sub _cmp_obj {
	my ($a, $b, $m) = @_;

	foreach my $f ( @{$m} ) {
		if ((defined $a->$f && defined $b->$f && !($a->$f cmp $b->$f) ) ||
			(! defined $a->$f && ! defined $b->$f)) {
			next 
		} else {
			return 1
		}
	}

	return 0;
}

sub uniq_obj {
	my ($obj, $list, $method_list) = @_;

	foreach my $obj2 ( @{$list} ) {
		if (! &_cmp_obj($obj, $obj2, $method_list)) {
			return 0;
		}
	}

	return 1;
}

1;

__END__

=head1 NAME

Security::util::uniq - Methods to guarantee unique lists.

=head1 SYNOPSIS

@uniq = &uniq(@array);
@uniq = &uniq_obj($obj, $list, $method_list);

=head1 DESCRIPTION

Methods to return only the unique values in an array or objects.

=head1 API

It provides the following API calls for other modules to use.

=over 4

=item C<uniq(@array)>

Uses perl hash keys to return only the unique elements in the array,
order is B<not> guaranteed.

=back

=over 4

=item C<uniq_obj($obj, $list, $method_list)>

Verify that C<$obj> is unique in C<$list> using all the methods listed
in C<$method_list> 

=back

=head1 SEE ALSO

L<sort>

=cut
